unit UDP;

interface

uses
 Windows,Messages,Winsock,SysUtils;

const
 UDP_BASE_ERROR=0;
 UDP_NO_SOCKET =UDP_BASE_ERROR+1;
 UDP_LAST_ERROR=UDP_BASE_ERROR+1;

type
 TCustomUDPClient=class
 protected
  fError :integer;
  fSocket:integer;
  fFrom  :TSockAddr;
  procedure SetError(Value:integer);
  procedure FatalError(ErrNo:integer);
  procedure OpenSocket; virtual;
  procedure BindSocket(Port:word);
  property Error:integer read fError write SetError;
 public
  constructor Create;
  destructor Destroy; override;
  procedure Close; virtual;
  property From:TSockAddr read fFrom;
  function Read:string;
 end;

procedure WaitOrKill(var Thread:THandle; Time:cardinal);
procedure LSwap(var c:cardinal);
function GetLocalAddr:integer;
function INetAddr(Host:string):integer;
function Dump(s:string):string;
function CreateGUID:string;
function CreateID:string;

implementation

//----------------------------------------------------------------------------//

procedure WaitOrKill(var Thread:THandle; Time:cardinal);
begin
 if Thread=0 then exit;
 if WaitForSingleObject(Thread,Time)=WAIT_TIMEOUT then begin
  TerminateThread(Thread,0);
  CloseHandle(Thread);
  Thread:=0;
 end;
end;

procedure LSwap(var c:cardinal);
begin
 c:=swap(c shr 16)+swap(c) shl 16;
end;

function GetLocalAddr:integer;
var
 host:array[0..80] of char;
begin
 if gethostname(host,sizeof(host))=0 then
  Result:=INetAddr(host)
 else
  Result:=INADDR_NONE;
end;

function INetAddr(Host:string):integer;
var
 pHost:PChar;
 HostEnt:PHostEnt;
begin
 if Host='' then begin
  Result:=INADDR_NONE;
 end else begin
  pHost:=PChar(Host);
  Result:=inet_addr(pHost);
  if Result=INADDR_NONE then begin
   HostEnt:=gethostbyname(pHost);
   if HostEnt<>nil then Result:=integer(pointer(HostEnt^.h_addr^)^);
  end;
 end;
end;

function Dump(s:string):string;
const
 hx:array[0..$F] of char='0123456789ABCDEF';
var
 i:integer;
 b:byte;
begin
 SetLength(Result,2*Length(s));
 for i:=1 to Length(s) do begin
  b:=ord(s[i]);
  Result[2*i-1]:=hx[b shr  4];
  Result[2*i  ]:=hx[b and $f];
 end;
end;

function CoCreateGuid(out guid: TGUID): HResult; stdcall; external 'ole32.dll';

function CreateGUID:string;
var
 guid:TGUID;
begin
 SetLength(Result,SizEOf(guid));
 move(guid,Result[1],SizeOf(guid));
end;

function CreateID:string;
begin
 Result:=Dump(CreateGUID);
end;

//----------------------------------------------------------------------------//

constructor TCustomUDPClient.Create;
begin
 fSocket:=INVALID_SOCKET;
end;

destructor TCustomUDPClient.Destroy;
begin
 Close;
 inherited;
end;

procedure TCustomUDPClient.SetError(Value:integer);
begin
 if Value=SOCKET_ERROR then begin
  FatalError(WSAGetLastError);
 end;
end;

procedure TCustomUDPClient.FatalError(ErrNo:integer);
begin
 fError:=ErrNo;
 Abort;
end;

procedure TCustomUDPClient.OpenSocket;
begin
 Close;
 fSocket:=socket(AF_INET,SOCK_DGRAM,IPPROTO_UDP);
 if fSocket=INVALID_SOCKET then FatalError(UDP_NO_SOCKET);
end;

procedure TCustomUDPClient.Close;
begin
 if fSocket<>INVALID_SOCKET then begin
  closesocket(fSocket);
  fSocket:=INVALID_SOCKET;
 end;
end;

procedure TCustomUDPClient.BindSocket(Port:word);
var
 sa:TSockAddr;
begin
 FillChar(sa,SizeOf(sa),0);
 sa.sin_family:=AF_INET;
 sa.sin_port:=htons(Port);
 Error:=bind(fSocket,sa,SizeOf(sa));
end;

function TCustomUDPClient.Read:string;
var
 l,i:integer;
begin
 FillChar(fFrom,SizeOf(fFrom),0);
 fFrom.sin_family:=AF_INET;
 l:=1500;
 SetLength(Result,l);
 i:=SizeOf(fFrom);
 l:=recvfrom(fSocket,Result[1],l,0,fFrom,i);
 if l<=0 then Result:='' else SetLength(Result,l);
end;

//----------------------------------------------------------------------------//

procedure WSAStartup;
var
 wsa:TWSAData;
begin
 Winsock.WSAStartup($101,wsa);
end;

initialization
 WSAStartup;
end.

